@@ -8,6 +8,14 @@
  *	Revision Log
  *
 $Log: build.c,v $
+ * Revision 1.9  1993/05/16  23:03:15  moore
+ * HeNCE v1.4
+ * + add pvm3 compilation support
+ *
+ * Revision 1.8  1992/12/10  03:46:24  moore
+ * * errno patch for CRAY.
+ * * declare index() for old CRAY systems.
+ *
  * Revision 1.7  1992/12/09  04:57:09  moore
  * in built makefile: when doing make install, use cp instead of mv.
  * mv fails when doing a make on more than one machine, if the machines
@@ -703,8 +711,13 @@ XtPointer cd;
 	int i;
 	char buf[256];
 	static char *libnames[] = {
+#ifdef PVM3
+		"$(HLIBDIR)/libslave3.a",
+		"$(HLIBDIR)/libhence3.a",
+#else
 		"$(HLIBDIR)/libslave.a",
 		"$(HLIBDIR)/libhence.a",
+#endif
 		"$(HLIBDIR)/librb.a",
 		"$(HLIBDIR)/libdl.a",
 		"$(HLIBDIR)/liballoc.a",
@@ -771,7 +784,11 @@ XtPointer cd;
 
 	FP (ff, "# The following defines are set from these X resources:\n");
 	FP (ff, "# HLIBDIR\thtool.henceLibDir\n");
+#ifdef PVM3
+	FP (ff, "# PLIB\thtool.pvm3Lib\n");
+#else
 	FP (ff, "# PLIB\thtool.pvmLib\n");
+#endif
 	FP (ff, "# XDIR\thtool.binDir\n");
 	FP (ff, "# INCLUDES\thtool.includeDir\n");
 	FP (ff, "HLIBDIR\t=\t%s\n", henceLibDir);
@@ -811,7 +828,11 @@ XtPointer cd;
 	FP (ff, "# CDEBUGFLAGS or just type \"make CDEBUGFLAGS=whatever\"\n");
 	FP (ff, "#\n");
 	FP (ff, "#CDEBUGFLAGS=-g\n");
+#ifdef PVM3
+	FP (ff, "CFLAGS\t=\t-DIMA_$(ARCH) -DPVM3 $(CDEBUGFLAGS)\n");
+#else
 	FP (ff, "CFLAGS\t=\t-DIMA_$(ARCH) $(CDEBUGFLAGS)\n");
+#endif
     FP (ff, "\n");
 	FP (ff, "PROGRAMS=");
 	for (tn = rb_First(st); tn != st; tn = rb_Next(tn))
