@@ -9,6 +9,11 @@
  *	Revision Log
  *
 $Log: costmat.c,v $
+ * Revision 1.6  1993/05/16  23:03:49  moore
+ * add hack to host file generation needed by pvm3
+ * (even though pvm3 doesn't use it yet)
+ * HeNCE v1.4
+ *
  * Revision 1.5  1992/12/09  04:58:33  moore
  * don't write out an entry from cost matrix if cost == 0
  *
@@ -869,6 +874,16 @@ struct costm *cm;
 	int have_localhost;
 	char myhostname[1024];
 	char line[1024];
+	/*
+	 * This flag says whether we have seen an asterisk in the
+	 * user-supplied hosts file.  If we haven't seen one, don't output
+	 * one before supplying entries that are in the cost matrix but
+	 * not in the hosts file.  Some versions of pvm choke on asterisks.
+	 *
+	 * This is named after Nathan Hale who said...
+	 * "I regret that I have but one asterisk for my country".
+	 */
+	int nathan = 0;
 
 	/*
 	 * for each host in the cost matrix, reset the "host is needed"
@@ -923,6 +938,7 @@ struct costm *cm;
 				continue;
 			else if (*line == '*') {
 				fprintf (out, "%s", line);
+				nathan = 1;
 				continue;
 			}
 			if (write_host_if_needed (out, hostAddrList, line, cm))
@@ -935,7 +951,9 @@ struct costm *cm;
 	 * that host's name to the pvm host file.  Finally, if we haven't
 	 * output the name of the local host, do that last.
 	 */
-	fprintf (out, "*\n");		/* disable any defaults */
+	if (nathan)
+		fprintf (out, "*\n");		/* disable any defaults */
+
 	for (h = 0; h < cm->nhost; ++h) {
 		if ((cm->hosts[h] == NULL) ||
 			(cm->hflag[h] & SUBHOST_HOST) == 0 ||
