@@ -8,6 +8,14 @@
  *	Revision Log
  *
 $Log: execute.c,v $
+ * Revision 1.3  1993/05/16  23:04:38  moore
+ * HeNCE 1.4
+ * + PVM3 support: different startup code for pvm console
+ *
+ * Revision 1.2  1992/12/10  03:47:23  moore
+ * errno hack for CRAY.
+ * Also declare index() for old CRAY O.S.
+ *
  * Revision 1.1  1992/04/08  05:48:42  moore
  * initial RCS version
  *
@@ -191,6 +199,20 @@ do_execute()
 		msg_Format ("(if you've added/deleted hosts you need to kill pvm and restart)\n");
 	}
 	else {
+#ifdef PVM3
+		/*
+		 * we can't pass a hosts file to the pvm console anyway,
+		 * so don't bother for now.
+		 *
+		 * XXX to do: start up pvmd3 and pass *it* the hosts file,
+		 * then start up the console.  We don't have to wait until
+		 * pvmd3 is up before running the console; the executioner
+		 * adds its hosts anyway.
+		 */
+		msg_Format ("starting up pvm...\n");
+		if (pvm_Start ("") < 0)
+			return;
+#else
 		/*
 		 * build a "temporary" pvm hosts file, containing only those
 		 * hosts necessary to run this HeNCE program, from the master
@@ -224,11 +246,11 @@ do_execute()
 		}
 		fclose (srcfp);
 		fclose (dstfp);
-
 		msg_Format ("starting up pvm...\n");
 		if (pvm_Start (tmpPvmHostFile) < 0)
 			return;
 		/* XXX need to wait until pvm is up and running ... */
+#endif PVM3
 	}
 
 	sprintf (crud, "exec master %s -tf %s -cm %s",
