@@ -508,6 +508,16 @@ int x, y;
 	n->xy.y = y;
 }
 
+static int
+paramcmp (k1, k2)
+Key *k1, *k2;
+{
+	register char *c1, *c2;
+	c1 = k1->pkey;
+	c2 = k2->pkey;
+	return strcmp (c1, c2);
+}
+
 void
 gr_AddNodeParam (n, p)
 Node n;
@@ -517,7 +527,7 @@ Param p;
 	int strcmp ();
 
 	k.pkey = (void *) (p->name);		/* XXX should this be malloc'ed? */
-	rb_Insert (n->params, &k, (void *) p, strcmp);
+	rb_Insert (n->params, &k, (void *) p, paramcmp);
 }
 
 /*	gr_PatchNode()
