@@ -8,6 +8,13 @@
  *	Revision Log
  *
  $Log: pvmglue.c,v $
+ * Revision 1.5  1993/05/16  23:06:13  moore
+ * HeNCE 1.4 support
+ * + pvm3 support
+ *
+ * Revision 1.4  1992/12/10  03:49:31  moore
+ * declare index() for old CRAY systems.
+ *
  * Revision 1.3  1992/12/02  22:21:56  moore
  * if pvm exits with non zero status, user is prompted to press
  * return before window disappears.  (ugly hack)
@@ -45,6 +52,15 @@ extern int errno;				/* 4.3bsd needs this */
 #endif
 
 static int pvmPid = 0;
+#ifdef PVM3
+static int pvmConsolePid = 0;
+#endif
+
+#ifdef PVM3
+char pvmd_name[] = "pvmd3";
+#else
+char pvmd_name[] = "pvmd";
+#endif
 
 static void
 pvm_Done (pid, exitArg, clientData, howDied)
@@ -54,27 +70,23 @@ void *clientData;
 enum causeOfDeath howDied;
 {
     if (howDied == SUBPROC_KILLED) {
-		msg_Format ("pvmd (pid %d) was killed with signal %d\n",
-					pid, exitArg & ~0200);
+		msg_Format ("%s (pid %d) was killed with signal %d\n",
+					pvmd_name, pid, exitArg & ~0200);
     }
     else if (exitArg != 0) {
-		msg_Format ("pvmd (pid %d) exited abnormally with status %d\n",
-					pid, exitArg);
+		msg_Format ("%s (pid %d) exited abnormally with status %d\n",
+					pvmd_name, pid, exitArg);
     }
     else
-		msg_Format ("pvmd exited normally\n");
+		msg_Format ("%s exited normally\n", pvmd_name);
 
     if (pid == pvmPid) {	/* should always be true */
 		subproc_Pclose (pvmPid);
 		pvmPid = 0;
 		if (clientData != NULL) {
-#if 0
-			set_label ((Widget) clientData, "start pvm");
-#else
 			XtVaSetValues ((Widget) clientData,
 						   XtNsensitive, True,
 						   NULL);
-#endif
 		}
     }
 }
@@ -134,8 +146,8 @@ char *hostfile;
 				 * child died without our knowing it - do appropriate
 				 * cleanup.
 				 */
-				msg_Format ("OOPS: pvmd (pid %d) died without SIGCHLD\n",
-							pvmPid);
+				msg_Format ("OOPS: %s (pid %d) died without SIGCHLD\n",
+							pvmd_name, pvmPid);
 				pvm_Done (pvmPid, 0, startPvmButton, SUBPROC_EXITED);
 			}
 		}
@@ -171,27 +183,28 @@ char *hostfile;
     if (pvmPid != 0)
 		return 1;
 
+#ifdef PVM3
+	msg_Format ("running %s\n", pvmdExecPath);
+	sprintf (crud,
+			 "exec %s -title 'pvm3 console' -sb -e /bin/sh -c '%s || (echo Press return to exit.; read x)'",
+			 xtermCommand, pvmdExecPath);
+#else
 	msg_Format ("running %s -i %s\n", pvmdExecPath, hostfile);
-
     sprintf (crud,
-			 "exec %s -title 'HeNCE - PVM console' -sb -e /bin/sh -c '%s -i %s || (echo Press return to exit.; read x)'",
+			 "exec %s -title 'HeNCE - PVM console' -sb -e /bin/sh -c '%s -i %s ; (echo Press return to exit.; read x)'",
 			 xtermCommand, pvmdExecPath, hostfile);
-#if 0
-	fprintf (stderr, "%s\n", crud);
 #endif
+
     if ((pvmPid = subproc_Popen (crud, pvm_Done,
 								 pvm_Done, pvm_Sink, startPvmButton)) <= 0) {
 		msg_Format ("couldn't run %s\n", pvmdExecPath);
 		pvmPid = 0;
 		return -1;
     }
-#if 0
-    set_label (startPvmButton, "kill pvm");
-#else
+
 	XtVaSetValues (startPvmButton,
 				   XtNsensitive, False,
 				   NULL);
-#endif
     return 0;
 }
 
@@ -201,8 +214,6 @@ pvm_IsRunning ()
     return (pvmPid != 0);
 }
 
-
-
 /*
  * Local variables:
  * tab-width:4
