@@ -7,7 +7,10 @@
  *
  *	Revision Log
  *
-$Log$
+$Log: symtab.h,v $
+ * Revision 1.2  1992/04/08  06:50:01  moore
+ * incorporates changes up to 8 April 1992
+ *
  *
  */
 
@@ -23,3 +26,4 @@ extern int st_Lookup();
 
 extern int st_Insert();
 
+extern char *st_Key ();
