@@ -7,7 +7,10 @@
  *
  *	Revision Log
  *
-$Log$
+$Log: xcomn.h,v $
+ * Revision 1.1  1992/04/08  05:50:52  moore
+ * initial RCS version
+ *
  *
  */
 
