@@ -8,6 +8,11 @@
  *	Revision Log
  *
 $Log: xsetup.c,v $
+ * Revision 1.5  1993/05/16  23:07:03  moore
+ * HeNCE 1.4
+ * + add pvm3 support: changed a few resource defs.
+ * + print out which version of pvm we're using
+ *
  * Revision 1.4  1992/09/18  01:56:28  moore
  * add #include <sys/stat.h> to satisfy somebody
  *
@@ -98,15 +103,25 @@ static char *fallbacks[] = {
 	 * XXX some of these need to be initialized at compile time
 	 * based on configuration-specific data.
 	 */
+#ifdef PVM3
+	"*binDir: $(HOME)/pvm3/bin/$(ARCH)",
+#else
 	"*binDir: $(HOME)/pvm/$(ARCH)",
+#endif
 	"*editorCommand: xterm -title %s -e vi %s",
 	"*henceLibDir: $(HOME)/hence/$(ARCH)",
 	"*iconList: ~/htool_icons",
 	"*includeDir: $(HOME)/hence/include",
 	"*language: C",
+#ifdef PVM3
+	"*pvmd3: $HOME/pvm3/lib/%s/pvm", /* XXX rationalize pvmd3 path stuff */
+	"*pvm3HostFile: /dev/null",
+	"*pvm3Lib: $(HOME)/pvm3/lib/$(ARCH)/libpvm3.a",
+#else
 	"*pvmd: /tmp/pvm/pvmd",
 	"*pvmHostFile: /dev/null",
 	"*pvmLib: $(HOME)/pvm/$(ARCH)/libpvm.a",
+#endif
 	/* "*reverseVideo:off", */
 	/* "*subDefsFile: sub.defs", */
 	"*traceFile: hence.trace",
@@ -163,12 +178,21 @@ static XtResource res_list[] = {
 		  XtOffset(app_resp_t, language), XtRString, "" },
 	{ "monochrome", "Monochrome", XtRBool, sizeof(Bool),
 		  XtOffset(app_resp_t, mono), XtRString, "off" },
+#ifdef PVM3
+	{ "pvmd3", "Pvmd3", XtRString, sizeof (String),
+		  XtOffset(app_resp_t, pvmdExecPath), XtRString, "" },
+	{ "pvm3HostFile", "Pvm3HostFile", XtRString, sizeof (String),
+		  XtOffset(app_resp_t, pvmHostFile), XtRString, "" },
+	{ "pvm3Lib", "Pvm3Lib", XtRString, sizeof (String),
+		  XtOffset(app_resp_t, pvmLib), XtRString, "" },
+#else
 	{ "pvmd", "Pvmd", XtRString, sizeof (String),
 		  XtOffset(app_resp_t, pvmdExecPath), XtRString, "" },
 	{ "pvmHostFile", "PvmHostFile", XtRString, sizeof (String),
 		  XtOffset(app_resp_t, pvmHostFile), XtRString, "" },
 	{ "pvmLib", "PvmLib", XtRString, sizeof (String),
 		  XtOffset(app_resp_t, pvmLib), XtRString, "" },
+#endif
 	{ "reverseVideo", "ReverseVideo", XtRBool, sizeof(Bool),
 		  XtOffset(app_resp_t, rvideo), XtRString, "off" },
 	{ "subDefsFile", "SubDefsFile", XtRString, sizeof (String),
@@ -306,7 +330,12 @@ Other options can be set via the X resource database, xrdb\n");
 	iconList = app_res.iconList;			/* XXX */
 	includeDir = strsave (texpand (app_res.includeDir));		/* XXX */
 	languageString = app_res.language; 
+#ifdef PVM3
+	sprintf (crud, app_res.pvmdExecPath, ARCHSTR);
+	pvmdExecPath = strsave (crud); /* XXX */
+#else
 	pvmdExecPath = app_res.pvmdExecPath;	/* XXX */
+#endif
 	pvmHostFile = strsave (texpand (app_res.pvmHostFile));		/* XXX */
 	pvmLib = strsave (texpand (app_res.pvmLib));				/* XXX */
 	if (app_res.subDefsFile && *app_res.subDefsFile)
@@ -334,7 +363,11 @@ Other options can be set via the X resource database, xrdb\n");
 	 * can't do this until after we've initialized the X stuff.
 	 */
 	msg_SetWriteFunction (message);
-    msg_Format ("HeNCE tool v%s\n", HTOOL_VERSION);
+#ifdef PVM3
+    msg_Format ("HeNCE tool v%s (for pvm 3.x)\n", HTOOL_VERSION);
+#else
+    msg_Format ("HeNCE tool v%s (for pvm 2.4.x)\n", HTOOL_VERSION);
+#endif
 
 	SetTraceFileLabel ();		/* in widmain.c */
 
