@@ -1,7 +1,7 @@
 /*
  * $Source: /a/thud/chalk/homes/moore/src/hence/master/RCS/anc.c,v $
- * $Revision: 1.1 $
- * $Date: 1992/04/08 05:51:36 $
+ * $Revision: 1.2 $
+ * $Date: 1993/05/16 23:13:09 $
  * $Author: moore $
  *
  * Anc.c defines routines to deal with getting information from ancestor 
@@ -147,7 +147,11 @@ Param ap, p;
   aa = talloc(array_anc, 1);
   aa->anc = (void *) a;
   aa->sub_name = a->sub_name;
+#ifdef PVM3
+  aa->pvmtid = a->pvmtid;
+#else
   aa->pvminum = a->pvminum;
+#endif
   aa->apnum = ap->num;
   aa->mypnum = p->num;
   return aa;
