@@ -1,7 +1,7 @@
 /*
  * $Source: /a/thud/chalk/homes/moore/src/hence/master/RCS/cost.c,v $
- * $Revision: 1.1 $
- * $Date: 1992/04/08 05:51:45 $
+ * $Revision: 1.2 $
+ * $Date: 1993/05/16 23:13:45 $
  * $Author: moore $
  */
 
@@ -107,6 +107,10 @@ Graph G;
     /* Read a line in the cost matrix: <mach-name> <sub-name> <cost> */
 
     nm = fscanf(f, "%s %s %d\n", mname, sname, &c);
+
+    if (*mname == '#')		/* ignore comments */
+	continue;
+
     if (nm == EOF) {
       return;
     } else if (nm != 3) {
@@ -154,7 +158,6 @@ Graph G;
               mname, sname);
       bail(CNULL);
     }
-
   }
 }
 
