@@ -1,7 +1,7 @@
 /*
  * $Source: /a/thud/chalk/homes/moore/src/hence/master/RCS/execute.c,v $
- * $Revision: 1.3 $
- * $Date: 1992/12/09 05:17:28 $
+ * $Revision: 1.4 $
+ * $Date: 1993/05/16 23:14:42 $
  * $Author: moore $
  */
 
@@ -14,7 +14,9 @@
 #include "rbhelp.h"
 #include "eval.h"
 
-#define BKM
+#ifdef PVM3
+#include <pvm3.h>
+#endif
 
 #ifdef IMA_ALPHA
 #define MAX_IDS_IN_TABLE    1000
@@ -61,17 +63,36 @@ Graph G;
   Readyq = G->readyq;
   set_master(G->alive);
 
-#ifdef BKM
-  /* BKM emit a list of to trace file */
+  /* emit a list of machines to trace file */
   rb_traverse(r,G->machines) trace_machine (r->k.key);
-#endif
 
   /* Fire up pvm */
 
   G->pvmproc = "master";
+#ifdef PVM3
+  /*
+   * XXX do we need to register our tid as "master" so others
+   * (maybe the HeNCE executioner) can find us?
+   */
+  G->pvmtid = pvm_mytid ();
+  if (G->pvmtid < 0) {
+      fprintf (stderr, "Can't talk to pvmd: pvm_mytid() returned %d [%s]\n",
+	       G->pvmtid, pvm_strerror (G->pvmtid));
+      nice_bail (CNULL);
+  }
+  trace_enroll (G->pvmproc, G->pvmtid);
+  rb_traverse (r, G->machines) {
+    if (add_machine (r->k.key) < 0) {
+      fprintf (stderr, "ERROR: can't add machine %s to PVM\n", r->k.key);
+      pvm_exit ();
+      bail (CNULL);
+    }
+  }
+#else
   G->pvminum = enroll(G->pvmproc);
   if (G->pvminum < 0) pvm_enroll_error(G->pvmproc, G->pvminum);
   trace_enroll(G->pvmproc, G->pvminum);
+#endif
   pvmon();
 
   /* Put the nodes from the heads list onto the ready queue */
@@ -94,7 +115,11 @@ Graph G;
     } else if (G->numexecuting == 0) {
       master_death(G->alive);
       t = time(0) - t;
+#ifdef PVM3
+      pvm_exit ();
+#else
       leave();
+#endif
       trace_done(t);
       return;
 
@@ -170,13 +195,49 @@ Graph G;
   /* Fire up the pvm process */
 
   n->ttime = time(0);
+#ifdef PVM3
+  /*
+   * XXX to do:
+   * recover when spawn fails:
+   * - lack of resources -- start on another machine.
+   * - no such file -- mark this machine as having infinite cost for
+   *   this program, and look for another machine.
+   */
+  {
+      int x;
+      x = pvm_spawn (n->sub_name,		/* routine name */
+		     (char **) NULL,		/* arguments (yeech!) */
+		     PvmTaskHost, n->machine,	/* specify host */
+		     1,				/* num tasks */
+		     &(n->pvmtid));		/* array to recv tids */
+      if (x < 0) {
+	  fprintf (stderr, "ERROR %d: spawning \"%s\" on %s : %s\n",
+		   n->pvmtid, n->sub_name, n->machine,
+		   pvm_strerror (x));
+	  nice_bail (CNULL);
+      }
+  }
+  /*
+   * pvm3.0 bug: apparently pvm_spawn() always returns the number of
+   * procs we wanted, not the number we got...so we check here also.
+   */
+  if (n->pvmtid < 0) {
+      fprintf (stderr, "ERROR %d: spawning \"%s\" on %s : %s\n",
+	       n->pvmtid, n->sub_name, n->machine, pvm_strerror (n->pvmtid));
+      nice_bail (CNULL);
+  }
+  /* Send the initial message */
+
+  trace_node(n);
+  pvm_initsend (PvmDataDefault);
+#else
   n->pvminum = initiateM(n->sub_name, n->machine);
   if (n->pvminum < 0) pvm_init_error(n);
-
   /* Send the initial message */
 
   trace_node(n);
   initsend();
+#endif
 #ifdef IMA_ALPHA
   putint (unique_id (n));
 #else
@@ -203,8 +264,8 @@ Graph G;
   putint(n->nargs);
   for(i = 0; i < n->nargs; i++) put_exp(n->args[i]);
   put_aa(n);
-#if 0
-  snd(n->sub_name, n->pvminum, HENCE_INIT);
+#ifdef PVM3
+  send_or_gripe (n->sub_name, n->pvmtid, HENCE_INIT);
 #else
   send_or_gripe (n->sub_name, n->pvminum, HENCE_INIT);
 #endif
@@ -231,23 +292,37 @@ Graph G;
   Rb_node r, rn;
   Param p;
   Exp e;
-  int rcvtypes[2], type, len, inum;
+  int rcvtypes[2], type, len;
+#ifdef PVM3
+  int tid;
+  int bufid;
+#else
+  int inum;
   char proc[64];
-  
+#endif
 
   /* Receive the message */
 
   rcvtypes[0] = HENCE_DIE;
   rcvtypes[1] = HENCE_DONE;
+#ifdef PVM3
+  bufid = retro_rcvmulti (2, rcvtypes);
+  pvm_bufinfo (bufid, &len, &type, &tid);
+  if (type == HENCE_DIE) {
+      fprintf (stderr, "Received DIE message from [%x]\n", tid);
+      nice_bail (CNULL);
+  }
+#else
   rcvmulti(2, rcvtypes);
-
-  G->numexecuting--;
-
   rcvinfo(&len, &type, proc, &inum);
   if (type == HENCE_DIE) {
     fprintf(stderr, "Received DIE message from %s/%d\n", proc, inum);
     nice_bail(CNULL);
   }
+#endif
+
+  G->numexecuting--;
+
 
   /* Check to make sure that the node's information is consistent */
 
@@ -352,12 +427,16 @@ Graph G;
 
   free_anc_list(n->anc);
   if (!n->arrays && strcmp(n->sub_name, "null") != 0) {
+#ifdef PVM3
+    pvm_initsend (PvmDataDefault);
+#else
     initsend();
+#endif
 /*     printf("Master: sending DIE to %d/%d (%s/%d)\n", */
 /*             n->id, n->inst, n->sub_name, n->pvminum); */
 /*     fflush(stdout); */
-#if 0
-    snd(n->sub_name, n->pvminum, HENCE_DIE);
+#ifdef PVM3
+    send_or_gripe (n->sub_name, n->pvmtid, HENCE_DIE);
 #else
     send_or_gripe (n->sub_name, n->pvminum, HENCE_DIE);
 #endif
@@ -913,6 +992,7 @@ int inum;
   nice_bail(CNULL);
 }
 
+#ifndef PVM3
 pvm_init_error(n)
 Node n;
 {
@@ -921,6 +1001,7 @@ Node n;
           n->sub_name, n->machine, n->pvminum);
   nice_bail(CNULL);
 }
+#endif
 
 #ifdef IMA_ALPHA
 /* 20-Aug-1992 AFV: don't pass 64 bit addresses as 32 bit ints */
