@@ -4,8 +4,8 @@
  * instead of C-style arrays.
  *
  * $Source: /a/thud/chalk/homes/moore/src/hence/master/RCS/fslave.c,v $
- * $Revision: 1.2 $
- * $Date: 1992/12/09 05:18:20 $
+ * $Revision: 1.3 $
+ * $Date: 1993/05/16 23:16:25 $
  * $Author: moore $
  */
 
@@ -18,6 +18,10 @@
 #include "htypes.h"
 #include "eval.h"
 
+#ifdef PVM3
+#include <pvm3.h>
+#endif
+
 char ID_INST[17];
 char *sl_id()
 {
@@ -26,10 +30,17 @@ char *sl_id()
 
 fslave()
 {
-    int inum, i, j;
+    int i, j;
+#ifdef PVM3
+	int mastid;
+	int tid;
+	int buflen, buftype;
+#else
     char master[64];
     int masinum;
+	int inum;
     char proc[64];
+#endif
     Param parray;
     Exp *pvalarray;
     int nparam, nargs;
@@ -45,14 +56,24 @@ fslave()
     int stime, sbtime;
     
     stime = time(0);
+#ifdef PVM3
+	tid = pvm_mytid ();
+#else
     inum = enroll("null");		/* XXX shouldn't have to do this */
     whoami(proc, &inum);
+#endif
     exp_initialize();			/* XXX shouldn't link in exp stuff */
     
     (void) signal(SIGSEGV, segv_catch);
+#ifdef PVM3
+	bufid = pvm_recv (-1, HENCE_INIT);
+    pvm_bufinfo (bufid, &buflen, &buftype, &mastid);
+    set_slave (mastid);
+#else
     rcv(HENCE_INIT);
     rcvinfo(&i, &j, master, &masinum);
-    set_slave(master, masinum);
+    set_slave (master, masinum);
+#endif
 
 	/*
 	 * get stuff from master.  See execute.c, routine fire_up ()
@@ -168,7 +189,11 @@ fslave()
     /* For all output array parameters, complete their initialization */
     
     die = 1;
+#ifdef PVM3
+    pvm_initsend (PvmDataDefault);
+#else
     initsend();
+#endif
     putint(node_ptr);
     putint(node_id);
     putint(node_inst);
@@ -190,14 +215,18 @@ fslave()
 		}
     }
     
-#if 0
-    snd(master, masinum, HENCE_DONE);
+#ifdef PVM3
+    send_or_gripe (NULL, mastid, HENCE_DONE);
 #else
     send_or_gripe (master, masinum, HENCE_DONE);
 #endif
     
     if (!die) service_array_requests(parray);
+#ifdef PVM3
+	pvm_exit ();
+#else
     leave();
+#endif
 }
 
 /*
