@@ -115,7 +115,11 @@ extern Param new_param();
 typedef struct {	/* Information about ancestors holding arrays */
   void *anc;
   char *sub_name;
+#ifdef PVM3
+  int pvmtid;
+#else
   int pvminum;
+#endif
   int apnum;
   int mypnum;
 } array_anc;
@@ -176,7 +180,11 @@ typedef struct a_node {  /* Representation of a node */
   struct a_node *pair;  /* Pair nodes: LOOP-ENDLOOP, etc. */
   struct a_node *scope; /* For static error checking */
   int flags;		/* Random manipulative variable */
+#ifdef PVM3
+  int pvmtid;		/* PVM 3.x task id */
+#else
   int pvminum;    	/* Pvm instance number */
+#endif
   int state;            /* State (e.g. READY, RUNNING, DONE).  If the 
                            state is negative, it is the number of parents
                            that this node is waiting for. */
@@ -216,7 +224,11 @@ typedef struct graph {
   Rb_node subroutines;  /* Subroutine list: key=sub name, val=sub ptr */
 
   char *pvmproc;	/* Pvm process name */
+#ifdef PVM3
+  int pvmtid;		/* PVM 3.x task id */
+#else
   int pvminum;		/* Pvm instance number */
+#endif
 
   char *cost_mat;	/* Cost matrix file name */
   char *graph_file;	/* Graph file name ("-" for stdin) */
