@@ -1,7 +1,7 @@
 /*
  * $Source: /a/thud/chalk/homes/moore/src/hence/master/RCS/node.c,v $
- * $Revision: 1.1 $
- * $Date: 1992/04/08 05:53:10 $
+ * $Revision: 1.2 $
+ * $Date: 1993/05/16 23:17:08 $
  * $Author: moore $
  */
 
@@ -93,7 +93,11 @@ Node n;
   nn->state = 0;
   nn->anc = new_anc_list();
   nn->machine = CNULL;
+#ifdef PVM3
+  nn->pvmtid = -1;
+#else
   nn->pvminum = -1;
+#endif
   nn->mirror = nn;
   n->mirror = nn;
   nn->aa = make_dl();
