@@ -1,7 +1,7 @@
 /*
  * $Source: /a/thud/chalk/homes/moore/src/hence/master/RCS/pvmhelp.c,v $
- * $Revision: 1.2 $
- * $Date: 1992/12/09 05:19:09 $
+ * $Revision: 1.3 $
+ * $Date: 1993/05/16 23:17:39 $
  * $Author: moore $
  */
 
@@ -14,14 +14,22 @@
 #include "htypes.h"
 #include "rbhelp.h"
 
+#ifdef PVM3
+#include <pvm3.h>
+#endif
+
 void segv_catch()
 {
   bail("Caught a segv\n");
 }
 
 static int is_slave = 0;
+#ifdef PVM3
+static int Mas_tid;
+#else
 static char *Mas_name;
 static int Mas_num;
+#endif
 static Rb_node Alive = (Rb_node) 0;
 static int ispvmon = 0;
 
@@ -42,6 +50,15 @@ Rb_node a;
   Alive = a;
 }
 
+#ifdef PVM3
+set_slave(mas_tid)
+int mas_tid;
+{
+  Mas_tid = mas_tid;
+  is_slave = 1;
+  ispvmon = 1;
+}
+#else
 set_slave(mas_name, mas_num)
 char *mas_name;
 int mas_num;
@@ -51,6 +68,7 @@ int mas_num;
   is_slave = 1;
   ispvmon = 1;
 }
+#endif
 
 nice_bail(s)
 char *s;
@@ -60,16 +78,21 @@ char *s;
   if (s != CNULL) fprintf(stderr, s);
   (void) fflush(stderr);
   if (is_slave) {
-    initsend();
-#if 0
-    snd(Mas_name, Mas_num, HENCE_DIE);
+#ifdef PVM3
+    pvm_initsend (PvmDataDefault);
+    send_or_gripe (NULL, Mas_tid, HENCE_DIE);
 #else
+    initsend();
     send_or_gripe (Mas_name, Mas_num, HENCE_DIE);
 #endif
   } else {
     master_death(Alive);
   }
+#ifdef PVM3
+  if (ispvmon) pvm_exit ();
+#else
   if (ispvmon) leave();
+#endif
   (void) exit((int)1);
 }
 
@@ -83,21 +106,30 @@ char *s;
   if (is_slave) {
     fprintf(stderr, "\n\n%sBailing\n\n", sl_id());
     (void) fflush(stderr);
-    initsend();
-#if 0
-    snd(Mas_name, Mas_num, HENCE_DIE);
+#ifdef PVM3
+    pvm_initsend (PvmDataDefault);
+    send_or_gripe (NULL, Mas_tid, HENCE_DIE);
 #else
+    initsend();
     send_or_gripe (Mas_name, Mas_num, HENCE_DIE);
 #endif
+#ifdef PVM3
+    pvm_exit ();
+#else
     leave();
+#endif
     { char *a = (char *)0; a[0] = 0; }
 /*     exit(1); */
   } else {
     fprintf(stderr, "\n\nBailing\n\n");
     (void) fflush(stderr);
     if (ispvmon) {
-      leave();
-      master_death(Alive);
+#ifdef PVM3
+	pvm_exit ();
+#else
+	leave();
+#endif
+	master_death(Alive);
     }
     { char *a = (char *)0; a[0] = 0; }
   }
@@ -115,10 +147,11 @@ Rb_node alive;
 /*     printf("Sending Die message to %d/%d (%s/%d)\n", n->id, n->inst, */
 /*       n->sub_name, n->pvminum); */
 /*     fflush(stdout); */
-    initsend();
-#if 0
-    snd(n->sub_name, n->pvminum, HENCE_DIE);
+#ifdef PVM3
+    pvm_initsend (PvmDataDefault);
+    send_or_gripe (n->sub_name, n->pvmtid, HENCE_DIE);
 #else
+    initsend();
     send_or_gripe (n->sub_name, n->pvminum, HENCE_DIE);
 #endif
   }
@@ -127,13 +160,21 @@ Rb_node alive;
 putint(n)
 int n;
 {
+#ifdef PVM3
+  pvm_pkint (&n, 1, 1);
+#else
   putnint(&n, 1);
+#endif
 }
 
 int getint()
 {
   int n;
+#ifdef PVM3
+  pvm_upkint (&n, 1, 1);
+#else
   getnint(&n, 1);
+#endif
   return n;
 }
 
@@ -141,39 +182,63 @@ putfloat(n)
 float n;
 {
   float f = n;
+#ifdef PVM3
+  pvm_pkfloat (&f, 1, 1);
+#else
   putnfloat(&f, 1);
+#endif
 }
 
 float getfloat()
 {
   float n[1];
+#ifdef PVM3
+  pvm_upkfloat (n, 1, 1);
+#else
   getnfloat(n, 1);
+#endif
   return n[0];
 }
 
 putdouble(n)
 double n;
 {
-  putndfloat(&n, 1);
+#ifdef PVM3
+    pvm_pkdouble (&n, 1, 1);
+#else
+    putndfloat(&n, 1);
+#endif
 }
 
 double getdouble()
 {
-  double n[1];
-  getndfloat(n, 1);
-  return n[0];
+    double n[1];
+#ifdef PVM3
+    pvm_upkdouble (n, 1, 1);
+#else
+    getndfloat(n, 1);
+#endif
+    return n[0];
 }
 
 putbyte(n)
 char n;
 {
-  putbytes(&n, 1);
+#ifdef PVM3
+    pvm_pkbyte (&n, 1, 1);
+#else
+    putbytes(&n, 1);
+#endif
 }
 
 char getbyte()
 {
   char n;
+#ifdef PVM3
+  pvm_upkbyte (&n, 1, 1);
+#else
   getbytes(&n, 1);
+#endif
   return n;
 }
 
@@ -181,16 +246,28 @@ putlstring(s)
 char *s;
 {
   putint(strlen(s));
+#ifdef PVM3
+  pvm_pkstr (s);
+#else
   putstring(s);
+#endif
 }
 
 char *getlstring()
 {
   char *s;
   int len;
+#ifdef PVM3
+  pvm_upkint (&len, 1, 1);
+#else
   getnint(&len, 1);
+#endif
   s = talloc(char, len);
-  getstring(s);
+#ifdef PVM3
+  pvm_upkstr (s);
+#else
+  getstring (s);
+#endif
   return s;
 }
 
@@ -206,6 +283,15 @@ Array a;
     putint(a->indsize[i]);
   }
   putint(a->tsize);
+#ifdef PVM3
+  switch(a->type) {
+    case INT: pvm_pkint (a->v.i, a->tsize, 1); break;
+    case FLOAT: pvm_pkfloat (a->v.f, a->tsize, 1); break;
+    case CHAR: pvm_pkbyte (a->v.c, a->tsize, 1); break;
+    case DOUBLE: pvm_pkdouble (a->v.d, a->tsize, 1); break;
+    default: bail("INT ERROR: put_array: bad type\n"); break;
+  }
+#else
   switch(a->type) {
     case INT: putnint(a->v.i, a->tsize); break;
     case FLOAT: putnfloat(a->v.f, a->tsize); break;
@@ -213,6 +299,7 @@ Array a;
     case DOUBLE: putndfloat(a->v.d, a->tsize); break;
     default: bail("INT ERROR: put_array: bad type\n"); break;
   }
+#endif
 }
   
 Array get_array()
@@ -230,6 +317,29 @@ Array get_array()
     a->indsize[i] = getint();
   }
   a->tsize = getint();
+#ifdef PVM3
+  switch(a->type) {
+    case INT: 
+      a->v.i = talloc(int, a->tsize); 
+      pvm_upkint (a->v.i, a->tsize, 1); 
+      break;
+    case FLOAT: 
+      a->v.f = talloc(float, a->tsize); 
+      pvm_upkfloat (a->v.f, a->tsize, 1); 
+      break;
+    case CHAR: 
+      a->v.c = talloc(char, a->tsize); 
+      pvm_upkbyte (a->v.c, a->tsize, 1);
+      break;
+    case DOUBLE: 
+      a->v.d = talloc(double, a->tsize); 
+      pvm_upkdouble (a->v.d, a->tsize, 1); 
+      break;
+    default:
+      bail("INT ERROR: get_array: bad type\n"); break;
+      break;
+  }
+#else
   switch(a->type) {
     case INT: 
       a->v.i = talloc(int, a->tsize); 
@@ -251,6 +361,7 @@ Array get_array()
       bail("INT ERROR: get_array: bad type\n"); break;
       break;
   }
+#endif
   build_ptr_tree(a);
   return a;
 }
@@ -270,6 +381,18 @@ Exp e;
   switch (e->elt_type) {
   case 'P': putint(e->val.p->num); break;
   case 'c': 
+#ifdef PVM3
+    switch(e->type) {
+      case INT: putint(e->val.i); break;
+      case FLOAT: pvm_pkfloat (&(e->val.f), 1, 1); break;
+      case CHAR: putbyte(e->val.c); break;
+      case DOUBLE: putdouble(e->val.d); break;
+      case ARRAY: put_array(e->val.a); break;
+      default: fprintf(stderr, "PUT_EXP: Unknown 'c' type %d\n", e->type);
+               bail(CNULL);
+               break;
+      }
+#else
     switch(e->type) {
       case INT: putint(e->val.i); break;
       case FLOAT: putnfloat(&(e->val.f), 1); break;
@@ -280,6 +403,7 @@ Exp e;
                bail(CNULL);
                break;
       }
+#endif
       break;
   case 'p':
   case 't':
@@ -467,7 +591,11 @@ Node n;
     aa = (Array_anc) d->val;
     putint((int)aa->anc);
     putlstring(aa->sub_name);
+#ifdef PVM3
+    putint(aa->pvmtid);
+#else
     putint(aa->pvminum);
+#endif
     putint(aa->apnum);
     putint(aa->mypnum);
   }
@@ -485,7 +613,11 @@ Dlist get_aa()
     aa = talloc(array_anc, 1);
     aa->anc = (void *) i;
     aa->sub_name = getlstring();
+#ifdef PVM3
+    aa->pvmtid = getint();
+#else
     aa->pvminum = getint();
+#endif
     aa->apnum = getint();
     aa->mypnum = getint();
     dl_insert_b_aa(d, aa);
@@ -507,6 +639,38 @@ int x;
     return pvm_errlist[x];
 }
 
+#ifdef PVM3
+send_or_gripe (name, tid, code)
+char *name;
+int tid;
+int code;
+{
+    int foo;
+    extern int pvm_aerr;
+    int bar;
+
+    bar = pvm_serror (0);
+
+    if ((foo = pvm_send (tid, code)) < 0) {
+	if (code == HENCE_DIE && foo == PvmNoTask)
+	    ;
+	else {
+	    if (name)
+		fprintf (stderr,
+			 "pvm_send (%x,%d) [to \"%s\"] failed. (%s)\n",
+			 tid, code, name, pvm_strerror (foo));
+	    else
+		fprintf (stderr,
+			 "pvm_send (%x,%d) failed.  (%s)\n",
+			 tid, code, pvm_strerror (foo));
+	}
+    }
+    pvm_serror (bar);
+#if 0
+    abort ();
+#endif
+}
+#else
 #define PvmNoComp -8		/* from pvm2.4.1/src/pvmuser.h */
 
 send_or_gripe (name, inst, code)
@@ -530,4 +694,63 @@ int code;
     }
     pvm_aerr = bar;
 }
+#endif /*PVM3*/
 
+#ifdef PVM3
+/*
+ * implement pvm2-style rcvmulti() function
+ */
+
+static int *saved_types;
+static int num_saved_types;
+
+static int
+rcvmatch (mid, tid, code)
+int mid, tid, code;
+{
+    int i;
+    int tag, x;
+
+    if ((x = pvm_bufinfo (mid, (int *) 0, &tag, &tid)) < 0)
+	return x;
+    for (i = 0; i < num_saved_types; ++i)
+	if (saved_types[i] == tag)
+	    return 1;
+    return 0;
+}
+
+retro_rcvmulti (ntypes, types)
+int ntypes;
+int types[];
+{
+    int x;
+    int (*old)();
+
+    num_saved_types = ntypes;
+    saved_types = types;
+    old = pvm_recvf (rcvmatch);
+    x = pvm_recv (-1, -1);
+    pvm_recvf (old);
+    return x;
+}
+
+int
+add_machine (machine)
+char *machine;
+{
+    char *av[2];
+    int sv[2];
+    int x;
+
+    x = pvm_serror (0);
+    av[1] = NULL;
+    av[0] = machine;
+    if (pvm_addhosts (av, 1, sv) < 0) {
+	if (*sv == PvmDupHost)
+	    return 0;
+	else
+	    return *sv;
+    }
+    pvm_serror (x);
+}
+#endif
