@@ -1,7 +1,7 @@
 /*
  * $Source: /a/thud/chalk/homes/moore/src/hence/master/RCS/req.c,v $
- * $Revision: 1.3 $
- * $Date: 1992/12/09 05:20:06 $
+ * $Revision: 1.4 $
+ * $Date: 1993/05/16 23:18:26 $
  * $Author: moore $
  */
 
@@ -17,6 +17,10 @@
 #include "htypes.h"
 #include "rbhelp.h"
 
+#ifdef PVM3
+#include <pvm3.h>
+#endif
+
 typedef struct {
 	Param p;
 	int oth_pnum;
@@ -28,8 +32,12 @@ typedef array_request *Array_request;
 #define ARNULL ((Array_request)0)
 
 static Param Sending;
+#ifdef PVM3
+static int To_node_tid;
+#else
 static char *To_node_proc;
 static int To_node_inum;
+#endif
 
 void
 bail_while_sending(signo)
@@ -38,8 +46,13 @@ int signo;						/* not used */
 	fprintf(stderr, "%sERROR: Caught a segmentation violation:\n", sl_id());
 	fprintf(stderr, "%s  While trying to send the array parameter: ", sl_id());
 	fprint_param_brief(stderr, Sending);
+#ifdef PVM3
+	fprintf(stderr, "\n%s  to pvm process %x.\n%s\n", 
+			sl_id(), To_node_tid, sl_id());
+#else
 	fprintf(stderr, "\n%s  to pvm process %s/%d.\n%s\n", 
 			sl_id(), To_node_proc, To_node_inum, sl_id());
+#endif
 	fprintf(stderr, "%s  Hence has assumed that %s is ", 
 			sl_id(), Sending->name);
 	if (Sending->val == ENULL) {
@@ -67,8 +80,14 @@ service_array_requests(parray)
 Param parray;
 {
 	int msgtypes[2];
-	int len, type, inum;
+	int len, type;
+#ifdef PVM3
+	int tid;
+	int bufid;
+#else
+	int inum;
 	char proc[64];
+#endif
 	Array_request ar;
 	
 	Dlist reqs, d;
@@ -83,13 +102,22 @@ Param parray;
 	
 	while(1) {
 		reqs = make_dl();
+#ifdef PVM3
+		bufid = retro_rcvmulti (2, msgtypes);
+		pvm_bufinfo (bufid, &len, &type, &tid);
+#else
 		rcvmulti(2, msgtypes);
 		rcvinfo(&len, &type, proc, &inum);
+#endif
 		/*     printf("%sRcvd req for send (%d) from %s/%d\n",  */
 		/*         sl_id(), type, proc, inum);  */
 		/*     fflush(stdout); */
+#ifdef PVM3
+		To_node_tid = tid;
+#else
 		To_node_proc = proc;
 		To_node_inum = inum;
+#endif
 		if (type == HENCE_DIE) {
 			return;
 		}
@@ -99,18 +127,26 @@ Param parray;
 		
 		oldsig = signal(SIGSEGV, bail_while_sending);
 		/* Traverse the reqs list, and send out the junk */
+#ifdef PVM3
+		pvm_initsend (PvmDataDefault);
+#else
 		initsend();
+#endif
 		dl_traverse(d, reqs) {
 			ar = (Array_request) d->val;
 			pack_array_send(ar);
+#ifdef PVM3
+			trace_msg_send(ar->p->num, ar->oth_pnum, tid);
+#else
 			trace_msg_send(ar->p->num, ar->oth_pnum, proc, inum);
+#endif
 			free_ar(ar);
 		}
 		putint(-1);
 		/*     printf("%sSending HENCE_SEND to %s/%d\n", sl_id(), proc, inum); */
 		/*     fflush(stdout); */
-#if 0
-		snd(proc, inum, HENCE_SEND);
+#ifdef PVM3
+		send_or_gripe (NULL, tid, HENCE_SEND);
 #else
 		send_or_gripe (proc, inum, HENCE_SEND);
 #endif
@@ -133,12 +169,20 @@ Exp *pva;
 		aa = (Array_anc) d->val;
 		if (last_aa == (Array_anc) 0) {
 			last_aa = aa;
+#ifdef PVM3
+			pvm_initsend (PvmDataDefault);
+#else
 			initsend();
+#endif
 		}
 		if (aa->anc != last_aa->anc) {
 			finish_array_request(last_aa, parray, pva);
 			last_aa = aa;
+#ifdef PVM3
+			pvm_initsend (PvmDataDefault);
+#else
 			initsend();
+#endif
 		}
 		pack_array_request(aa, parray);
 	}
@@ -155,7 +199,11 @@ Param parray;
 	Param p;
 	int i;
 	
+#ifdef PVM3
+	trace_msg_req(aa->sub_name, aa->pvmtid, aa->apnum, aa->mypnum);
+#else
 	trace_msg_req(aa->sub_name, aa->pvminum, aa->apnum, aa->mypnum);
+#endif
 	putint(aa->apnum);
 	putint(aa->mypnum);
 	p = &(parray[aa->mypnum]);
@@ -177,12 +225,13 @@ Exp *pva;
 	
 	putint(-1);
 	/*   printf("%sSending req\n", sl_id()); */
-#if 0
-	snd(last_aa->sub_name, last_aa->pvminum, HENCE_REQ); 
+#ifdef PVM3
+	send_or_gripe (last_aa->sub_name, last_aa->pvmtid, HENCE_REQ); 
+	pvm_recv (-1, HENCE_SEND);
 #else
 	send_or_gripe (last_aa->sub_name, last_aa->pvminum, HENCE_REQ); 
-#endif
 	rcv(HENCE_SEND);
+#endif
 	/*   printf("%sReq recvd\n", sl_id()); */
 	/*   fflush(stdout); */
 	
