@@ -1,7 +1,7 @@
 /*
  * $Source: /a/thud/chalk/homes/moore/src/hence/master/RCS/slave.c,v $
- * $Revision: 1.1 $
- * $Date: 1992/04/08 05:53:23 $
+ * $Revision: 1.3 $
+ * $Date: 1993/05/16 23:18:48 $
  * $Author: moore $
  */
 
@@ -14,6 +14,10 @@
 #include "htypes.h"
 #include "eval.h"
 
+#ifdef PVM3
+#include <pvm3.h>
+#endif
+
 char ID_INST[17];
 char *sl_id()
 {
@@ -22,10 +26,18 @@ char *sl_id()
 
 slave()
 {
-  int inum, i, j;
+  int i, j;
+#ifdef PVM3
+  int mastid;
+  int bufid;
+  int tid;
+  int buflen, buftype;
+#else
   char master[64];
   int masinum;
+  int inum;
   char proc[64];
+#endif
   Param parray;
   Exp *pvalarray;
   int nparam, nargs;
@@ -41,14 +53,24 @@ slave()
   int stime, sbtime;
 
   stime = time(0);
+#ifdef PVM3
+  tid = pvm_mytid ();
+#else
   inum = enroll("null");
   whoami(proc, &inum);
+#endif
   exp_initialize();
 
   (void) signal(SIGSEGV, segv_catch);
+#ifdef PVM3
+  bufid = pvm_recv (-1, HENCE_INIT);
+  pvm_bufinfo (bufid, &buflen, &buftype, &mastid);
+  set_slave(mastid);
+#else
   rcv(HENCE_INIT);
   rcvinfo(&i, &j, master, &masinum);
   set_slave(master, masinum);
+#endif
   node_ptr = getint();
   node_id = getint();
   node_inst = getint();
@@ -151,7 +173,11 @@ slave()
   /* For all output array parameters, complete their initialization */
 
   die = 1;
+#ifdef PVM3
+  pvm_initsend (PvmDataDefault);
+#else
   initsend();
+#endif
   putint(node_ptr);
   putint(node_id);
   putint(node_inst);
@@ -173,13 +199,17 @@ slave()
     }
   }
 
-#if 0
-  snd(master, masinum, HENCE_DONE);
+#ifdef PVM3
+  send_or_gripe (NULL, mastid, HENCE_DONE);
 #else
   send_or_gripe (master, masinum, HENCE_DONE);
 #endif
 
   if (!die) service_array_requests(parray);
+#ifdef PVM3
+  pvm_exit ();
+#else
   leave();
+#endif
 }
 
