@@ -1,22 +1,22 @@
 /*
  * $Source: /a/thud/chalk/homes/moore/src/hence/master/RCS/trace.c,v $
- * $Revision: 1.3 $
- * $Date: 1992/12/09 05:20:28 $
+ * $Revision: 1.6 $
+ * $Date: 1993/05/16 23:19:12 $
  * $Author: moore $
  */
 
 #include <stdio.h>
 #include <ctype.h>
 #include <sys/types.h>
-#ifndef IMA_CRAY
-#include <sys/timeb.h>
-#else
+#if defined(IMA_CRAY) || defined(IMA_SGI)
 struct timeb {
     time_t time;
     unsigned short millitm;
     short timezone;
     short dstflag;
 };
+#else
+#include <sys/timeb.h>
 #endif
 #include "std.h"
 #include "rb.h"
@@ -272,6 +272,17 @@ trace_anc_off()
   tac = 0;
 }
 
+#ifdef PVM3
+trace_msg_req(topr, totid, top, myp)
+char *topr;
+int totid, top, myp;
+{
+  if (!trace_msgs) return;
+
+  fprintf(stderr, "%sReq. P%d from %x (%s) -> my P%d\n",
+          sl_id(), top, totid, topr, myp);
+}
+#else
 trace_msg_req(topr, toinum, top, myp)
 char *topr;
 int toinum, top, myp;
@@ -281,7 +292,18 @@ int toinum, top, myp;
   fprintf(stderr, "%sReq. P%d from %s/%d -> my P%d\n",
           sl_id(), top, topr, toinum, myp);
 }
+#endif
 
+#ifdef PVM3
+trace_msg_send(myp, top, totid)
+int totid, top, myp;
+{
+  if (!trace_msgs) return;
+
+  fprintf(stderr, "%sSend. my P%d to [%x] -> their P%d\n",
+          sl_id(), myp, totid, top);
+}
+#else
 trace_msg_send(myp, top, topr, toinum)
 char *topr;
 int toinum, top, myp;
@@ -291,7 +313,7 @@ int toinum, top, myp;
   fprintf(stderr, "%sSend. my P%d to %s/%d -> their P%d\n",
           sl_id(), myp, topr, toinum, top);
 }
-
+#endif
 
 static void
 flush_every_second (fp, timeb)
@@ -319,7 +341,8 @@ struct timeb *timeb;
     }
 }
 
-#if defined(IMA_RIOS) || defined(IMA_CRAY)
+#if defined(IMA_RIOS) || defined(IMA_CRAY) || defined(IMA_SGI) || \
+    defined(IMA_ALPHA)
 #include <sys/time.h>
 
 static void
