@@ -12,6 +12,9 @@
 #include "cmd.h"
 #define SRVR_ERR(x) (SRC_SRVR | (x))
 
+#ifdef PVM3
+#include "pvm3.h"
+#endif
 
 char **child_environ;
 
@@ -400,6 +403,14 @@ int sig;
  * at present, but can be used as a debugging hook.
  */
 
+#ifdef PVM3
+void
+send_reply (tid)
+int tid;
+{
+    pvm_send (tid, 0);
+}
+#else
 void
 send_reply (proc, inum)
 char *proc;
@@ -407,12 +418,25 @@ int inum;
 {
     snd (proc, inum, 0);
 }
-
+#endif
 /*
  * send a simple yes/no reply with a message
  * a printf-like interface would be better
  */
 
+#ifdef PVM3
+void
+reply (tid, code, msg)
+int tid;
+int code;
+char *msg;
+{
+    pvm_initsend (PvmDataDefault);
+    putint (code);
+    pvm_pkstr (msg);
+    send_reply (tid);
+}
+#else
 void
 reply (proc, inum, code, msg)
 char *proc;
@@ -425,33 +449,50 @@ char *msg;
     putstring (msg);
     send_reply (proc, inum);
 }
+#endif
 
 int 
 putint (i)
 int i;
 {
+#ifdef PVM3
+    return pvm_pkint (&i, 1, 1);
+#else
     return putnint (&i, 1);
+#endif
 }
 
 int
 putlongint (l)
 int l;
 {
+#ifdef PVM3
+    return pvm_pklong (&l, 1, 1);
+#else
     return putnlong (&l, 1);
+#endif
 }
 
 int
 getint (ip)
 int *ip;
 {
+#ifdef PVM3
+    return pvm_upkint (ip, 1, 1);
+#else
     return getnint (ip, 1);
+#endif
 }
 
 int
 getlongint (lp)
 long *lp;
 {
+#ifdef PVM3
+    return pvm_upklong (lp, 1, 1);
+#else
     return getnlong (lp, 1);
+#endif
 }
 
 /*
@@ -563,10 +604,16 @@ char **env;
  */
 
 void
+#ifdef PVM3
+do_spawn (tid, child_argv)
+int tid;
+char **child_argv;
+#else
 do_spawn (proc, inum, child_argv)
 char *proc;
 int inum;
 char **child_argv;
+#endif
 {
     int child_stdout[2];	/* pipe for child's stdout */
     int child_stderr[2];	/* pipe for child's stderr */
@@ -630,12 +677,21 @@ char **child_argv;
 	    if (FD_ISSET (child_stderr[0], &temp_fds)) {
 		nbytes = read (child_stderr[0], buf, sizeof (buf));
 		if (nbytes > 0) {
+#ifdef PVM3
+		    pvm_initsend (PvmDataDefault);
+#else
 		    initsend ();
+#endif
 		    /* putf("%i %i %*b", MSG_STDERR, nbytes, nbytes, buf); */
 		    putint (MSG_STDERR);
 		    putint (nbytes);
+#ifdef PVM3
+		    pvm_pkbyte (buf, nbytes, 1);
+		    send_reply (tid);
+#else
 		    putbytes (buf, nbytes);
 		    send_reply (proc, inum);
+#endif
 		}
 		else if (nbytes == 0) {
 		    close (child_stderr[0]);
@@ -645,12 +701,21 @@ char **child_argv;
 	    if (FD_ISSET (child_stdout[0], &temp_fds)) {
 		nbytes = read (child_stdout[0], buf, sizeof (buf));
 		if (nbytes > 0) {
+#ifdef PVM3
+		    pvm_initsend (PvmDataDefault);
+#else
 		    initsend ();
+#endif
 		    /* putf("%i %i %*b", MSG_STDOUT, nbytes, nbytes, buf); */
 		    putint (MSG_STDOUT);
 		    putint (nbytes);
+#ifdef PVM3
+		    pvm_pkbyte (buf, nbytes, 1);
+		    send_reply (tid);
+#else
 		    putbytes (buf, nbytes);
 		    send_reply (proc,inum);
+#endif
 		}
 		else if (nbytes == 0) {
 		    close (child_stdout[0]);
@@ -660,11 +725,19 @@ char **child_argv;
 	    if (FD_ISSET (signal_pipe[0], &temp_fds)) {
 		nbytes = read (signal_pipe[0], &foo, sizeof foo);
 		if (foo.pid == child_pid) {
+#ifdef PVM3
+		    pvm_initsend (PvmDataDefault);
+#else
 		    initsend ();
+#endif
 		    /* putf ("%i %i", MSG_CMDSTATUS, foo.status); */
 		    putint (MSG_PROC_EXIT);
 		    putint (foo.status);
+#ifdef PVM3
+		    send_reply (tid);
+#else
 		    send_reply (proc,inum);
+#endif
 		    break;
 		}
 	    }
@@ -682,8 +755,12 @@ char *msgbuf;
     int cmd;
     int fd = -1;
     enum {MODE_READ, MODE_WRITE, MODE_NEITHER} fdmode = MODE_NEITHER ;
+#ifdef PVM3
+    int tid;
+#else
     char proc[33];
     int inum;
+#endif
     char filename[1024];
     long filesize;
     long modtime;
@@ -696,67 +773,166 @@ char *msgbuf;
     int x;
 
     while (1) {
+#ifdef PVM3
+	if ((x = pvm_recv (-1, 0)) < 0 ||
+	    (x = pvm_bufinfo (x, NULL, NULL, &tid) < 0)) {
+	    break;
+	}
+#else
 	if ((x = rcv (0)) < 0 || (x = rcvinfo (NULL, NULL, proc, &inum) < 0)) {
 	    break;
 	}
+#endif
 
 	if (msgbuf != NULL && *msgbuf != '\0') {
 	    /* putf ("%i %i %*b", MSG_STDERR, strlen(msgbuf),
 	             strlen(msgbuf), msgbuf) */
 	    putint (MSG_STDERR);
 	    putint (strlen (msgbuf));
+#ifdef PVM3
+	    pvm_pkbyte (msgbuf, strlen (msgbuf), 1);
+	    send_reply (tid);
+	    reply (tid, MSG_SRVR_EXIT, "");
+	    pvm_exit ();
+#else
 	    putbytes (msgbuf, strlen (msgbuf));
 	    send_reply (proc, inum);
 	    reply (proc, inum, MSG_SRVR_EXIT, "");
 	    leave ();
+#endif
 	    exit (1);
 	}
 
 	getint (&cmd);
 	switch (cmd) {
 	case CMD_CREATE:
+#ifdef PVM3
+	    if ((x = pvm_upkstr (filename)) != 0 ||
+		(x = getlongint (&filesize)) != 0 ||
+		(x = getlongint (&modtime)) != 0) {
+		reply (tid, pvm_error_code (x), "invalid argument");
+	    }
+#else
 	    if ((x = getstring (filename)) != 0 ||
 		(x = getlongint (&filesize)) != 0 ||
-		(x = getlongint (&modtime)) != 0)
+		(x = getlongint (&modtime)) != 0) {
 		reply (proc, inum, pvm_error_code (x), "invalid argument");
-	    if (fd != -1)
+	    }
+#endif
+	    if (fd != -1) {
+#ifdef PVM3
+		reply (tid, SRVR_ERR(0), "fd already open");
+#else
 		reply (proc, inum, SRVR_ERR(0), "fd already open");
-	    if ((fd = open (filename, O_WRONLY|O_CREAT|O_TRUNC, 0600)) < 0)
+#endif
+	    }
+	    if ((fd = open (filename, O_WRONLY|O_CREAT|O_TRUNC, 0600)) < 0) {
+#ifdef PVM3
+		reply (tid, sys_error_code (errno), strerror (errno));
+#else
 		reply (proc, inum, sys_error_code (errno), strerror (errno));
+#endif
+	    }
 	    fdmode = MODE_WRITE;
+#ifdef PVM3
+	    reply (tid, REPL_OK, "");
+#else
 	    reply (proc, inum, REPL_OK, "");
+#endif
 	    break;
 	case CMD_OPEN:
-	    if ((x = getstring (filename)) != 0)
+#ifdef PVM3
+	    if ((x = pvm_upkstr (filename)) != 0) {
+		reply (tid, pvm_error_code (x), "invalid argument");
+	    }
+#else
+	    if ((x = getstring (filename)) != 0) {
 		reply (proc, inum, pvm_error_code (x), "invalid argument");
-	    if (fd != -1)
+	    }
+#endif
+	    if (fd != -1) {
+#ifdef PVM3
+		reply (tid, SRVR_ERR(0), "fd already open");
+#else
 		reply (proc, inum, SRVR_ERR(0), "fd already open");
-	    if ((fd = open (filename, O_RDONLY, 0)) < 0)
+#endif
+	    }
+	    if ((fd = open (filename, O_RDONLY, 0)) < 0) {
+#ifdef PVM3
+		reply (tid, sys_error_code (errno), strerror (errno));
+#else
 		reply (proc, inum, sys_error_code (errno), strerror (errno));
+#endif
+	    }
 	    fdmode = MODE_READ;
+#ifdef PVM3
+	    reply (tid, REPL_OK, "");
+#else
 	    reply (proc, inum, REPL_OK, "");
+#endif
 	    break;
 	case CMD_STAT:
-	    if ((x = getstring (filename)) != 0)
+#ifdef PVM3
+	    if ((x = pvm_upkstr (filename)) != 0) {
+		reply (tid, pvm_error_code (x), "invalid argument");
+	    }
+#else
+	    if ((x = getstring (filename)) != 0) {
 		reply (proc, inum, pvm_error_code (x), "invalid argument");
-	    if (stat (filename, &sbuf) < 0)
+	    }
+#endif
+	    if (stat (filename, &sbuf) < 0) {
+#ifdef PVM3
+		reply (tid, sys_error_code (errno), strerror (errno));
+#else
 		reply (proc, inum, sys_error_code (errno), strerror (errno));
-
+#endif
+	    }
+#ifdef PVM3
+	    pvm_initsend (PvmDataDefault);
+#else
 	    initsend ();
+#endif
 	    putint (REPL_STAT);
 	    putlongint (sbuf.st_size);
 	    putlongint (sbuf.st_mtime);
+#ifdef PVM3
+	    send_reply (tid);
+#else
 	    send_reply (proc, inum);
+#endif
 	    break;
 	case CMD_WRITE:
-	    if ((x = getint (&nbytes)) != 0)
+	    if ((x = getint (&nbytes)) != 0) {
+#ifdef PVM3
+		reply (tid, pvm_error_code (x), "invalid argument");
+#else
 		reply (proc, inum, pvm_error_code (x), "invalid argument");
-	    if (nbytes < 0 || nbytes > sizeof (buf))
+#endif
+	    }
+	    if (nbytes < 0 || nbytes > sizeof (buf)) {
+#ifdef PVM3
+		reply (tid, SRVR_ERR(0), "invalid argument");
+#else
 		reply (proc, inum, SRVR_ERR(0), "invalid argument");
-	    if (fd < 0 || fdmode != MODE_WRITE)
+#endif
+	    }
+	    if (fd < 0 || fdmode != MODE_WRITE) {
+#ifdef PVM3
+		reply (tid, SRVR_ERR(0), "fd is not open for writing");
+#else
 		reply (proc, inum, SRVR_ERR(0), "fd is not open for writing");
-	    if ((x = getbytes (buf, nbytes)) < 0)
+#endif
+	    }
+#ifdef PVM3
+	    if ((x = pvm_upkbyte (buf, nbytes, 1)) < 0) {
+		reply (tid, pvm_error_code (x), "buffer too small");
+	    }
+#else
+	    if ((x = getbytes (buf, nbytes)) < 0) {
 		reply (proc, inum, pvm_error_code (x), "buffer too small");
+	    }
+#endif
 	    if (nbytes == 0) {
 		/*
 		 * file is complete.  make sure size is correct.
@@ -768,16 +944,30 @@ char *msgbuf;
 		 * first to make sure things get flushed?
 		 */
 		if (fstat (fd, &sbuf) < 0) {
-		    reply (proc, inum, sys_error_code(errno), strerror (errno));
+#ifdef PVM3
+		    reply (tid, sys_error_code(errno), strerror (errno));
+#else
+		    reply (proc, inum, sys_error_code(errno),
+			   strerror (errno));
+#endif
 		    continue;
 		}
 		if (sbuf.st_size != filesize) {
+#ifdef PVM3
+		    reply (tid, SRVR_ERR(0), "incorrect size");
+#else
 		    reply (proc, inum, SRVR_ERR(0), "incorrect size");
+#endif
 		    close (fd);
 		    continue;
 		}
 		if (close (fd) < 0) {
-		    reply (proc, inum, sys_error_code(errno), strerror (errno));
+#ifdef PVM3
+		    reply (tid, sys_error_code(errno), strerror (errno));
+#else
+		    reply (proc, inum, sys_error_code(errno),
+			   strerror (errno));
+#endif
 		    fd = -1;
 		    continue;
 		}
@@ -791,32 +981,63 @@ char *msgbuf;
 		 */
 
 		if (setfiletime (filename, modtime) < 0) {
-		    reply (proc, inum, sys_error_code(errno), strerror (errno));
+#ifdef PVM3
+		    reply (tid, sys_error_code(errno), strerror (errno));
+#else
+		    reply (proc, inum, sys_error_code(errno),
+			   strerror (errno));
+#endif
 		    continue;
 		}
+#ifdef PVM3
+		reply (tid, REPL_OK, "");
+#else
 		reply (proc, inum, REPL_OK, "");
+#endif
 	    }
 	    else {
 		if (write (fd, buf, nbytes) != nbytes) {
-		    reply (proc, inum, sys_error_code(errno), strerror (errno));
+#ifdef PVM3
+		    reply (tid, sys_error_code(errno), strerror (errno));
+#else
+		    reply (proc, inum, sys_error_code(errno),
+			   strerror (errno));
+#endif
 		    close (fd);
 		    fd = -1;
 		}
-		else
+		else {
+#ifdef PVM3
+		    reply (tid, REPL_OK, "");
+#else
 		    reply (proc, inum, REPL_OK, "");
+#endif
+		}
 	    }
 	    break;
 	case CMD_READ:
 	    if ((x = getint (&nbytes)) != 0) {
+#ifdef PVM3
+		reply (tid, pvm_error_code (x), "invalid argument");
+#else
 		reply (proc, inum, pvm_error_code (x), "invalid argument");
+#endif
 		break;
 	    }
 	    if (nbytes < 0) {
+#ifdef PVM3
+		reply (tid, SRVR_ERR(0), "invalid argument");
+#else
 		reply (proc, inum, SRVR_ERR(0), "invalid argument");
+#endif
 		break;
 	    }
 	    if (fd < 0 || fdmode != MODE_READ) {
+#ifdef PVM3
+		reply (tid, SRVR_ERR(0), "fd is not open for reading");
+#else
 		reply (proc, inum, SRVR_ERR(0), "fd is not open for reading");
+#endif
 		break;
 	    }
 
@@ -825,7 +1046,11 @@ char *msgbuf;
 		nbytes = sizeof (buf);
 
 	    if ((nbytes = read (fd, buf, nbytes)) < 0) {
+#ifdef PVM3
+		reply (tid, sys_error_code(errno), strerror (errno));
+#else
 		reply (proc, inum, sys_error_code(errno), strerror (errno));
+#endif
 		close (fd);
 		fd = -1;
 		break;
@@ -833,48 +1058,90 @@ char *msgbuf;
 	    if (nbytes == 0) {
 		close (fd);
 		fd = -1;
+#ifdef PVM3
+		pvm_initsend (PvmDataDefault);
+#else
 		initsend ();
+#endif
 		putint (REPL_READ);
 		putint (0);
+#ifdef PVM3
+		send_reply (tid);
+#else
 		send_reply (proc, inum);
+#endif
 	    }
 	    else {
+#ifdef PVM3
+		pvm_initsend (PvmDataDefault);
+#else
 		initsend ();
+#endif
 		putint (REPL_READ);
 		putint (nbytes);
+#ifdef PVM3
+		pvm_pkbyte (buf, nbytes, 1);
+		send_reply (tid);
+#else
 		putbytes (buf, nbytes);
 		send_reply (proc, inum);
+#endif
 	    }
 	    break;
 	case CMD_SPAWN:
 	    if ((x = getint (&child_argc)) != 0) {
+#ifdef PVM3
+		reply (tid, pvm_error_code (x), "invalid argument");
+#else
 		reply (proc, inum, pvm_error_code (x), "invalid argument");
+#endif
 		break;
 	    }
 	    if (child_argc <= 0) {
+#ifdef PVM3
+		reply (tid, SRVR_ERR(0), "invalid argument");
+#else
 		reply (proc, inum, SRVR_ERR(0), "invalid argument");
+#endif
 		break;
 	    }
 	    j = 0;
 	    for (i = 0; i < child_argc; ++i) {
+#ifdef PVM3
+		if ((x = pvm_upkstr (buf)) < 0) {
+		    reply (tid, pvm_error_code (x), "out of data");
+		    break;
+		}
+#else
 		if ((x = getstring (buf)) < 0) {
 		    reply (proc, inum, pvm_error_code (x), "out of data");
 		    break;
 		}
+#endif
 		child_argv[j++] = expand_vars (buf, child_environ);
 	    }
 	    if (i == child_argc) {
 		child_argv[j] = NULL;
+#ifdef PVM3
+		do_spawn (tid, child_argv);
+#else
 		do_spawn (proc, inum, child_argv);
+#endif
 	    }
 	    for (i = 0; i < j; ++i)
 		if (child_argv[i])
 		    free (child_argv[i]);
 	    break;
 	case CMD_QUIT:
+#ifdef PVM3
+	    reply (tid, MSG_SRVR_EXIT, "");
+	    reply (tid, REPL_OK, "");
+	    pvm_exit ();
+#else
 	    reply (proc, inum, MSG_SRVR_EXIT, "");
 	    reply (proc, inum, REPL_OK, "");
 	    leave ();
+#endif
 	    exit (0);
 	default:
 	    break;
@@ -889,7 +1156,11 @@ char **argv;
 char **envp;
 {
     char foobar[1024];
+#ifdef PVM3
+    int tid;
+#else
     int inum;
+#endif
     char msgbuf[1024];
     int i, j;
     char **new_envp;
@@ -899,7 +1170,7 @@ char **envp;
      */
     for (i = 0; envp[i]; ++i);	/* count current number of entries  */
     i += 2;			/* leave room for more */
-    new_envp = malloc (i * sizeof (char *)); /* allocate space */
+    new_envp = (char **) malloc (i * sizeof (char *)); /* allocate space */
 
     /* copy existing entries, except for ARCH */
     j = 0;
@@ -912,11 +1183,17 @@ char **envp;
     new_envp[j] = NULL;
     child_environ = new_envp;
 
-    if (whoami (foobar, &inum) < 0) {
-	fprintf (stderr, "whoami () failed\n");
+#ifdef PVM3
+    if ((tid = pvm_mytid ()) < 0) {
+	fprintf (stderr, "pvm_mytid() failed .... is pvm 3.x running?\n");
 	exit (1);
     }
-
+#else
+    if (whoami (foobar, &inum) < 0) {
+	fprintf (stderr, "whoami () failed ... is pvm 2.x running?\n");
+	exit (1);
+    }
+#endif
     /*
      * set up the child sig handler stuff
      */
@@ -926,6 +1203,8 @@ char **envp;
     /*
      * cd to the right place before starting
      * create directory if necessary
+     *
+     * XXX for pvm3, is this the right directory?
      */
     sprintf (foobar, "%s/pvm/%s/tmp", getenv("HOME"), ARCHSTR);
     if (chdir (foobar) < 0) {
